#!/usr/bin/python
# 
# Copyright (C) 2009, Intel Corporation.
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms and conditions of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.  
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
# Place - Suite 330, Boston, MA 02111-1307 USA.
#
# Authors:
#              Wei, Zhang <wei.z.zhang@intel.com>
#

import sys
import os
import re

sys.path.append('../../tetj/')
from tetj import *

sys.path.append('../utils/')
from shell_exec import *

info = ""

#=======================================
# Print + Journal
def __P(log):

    global info

    sys.stdout.write(log)
    info += log

#=======================================
# Parse package version from one string
def __parse_pkg_version(pkg_ver_str):
    pkg_ver_str = "\n" + pkg_ver_str
    res = re.findall(r'[-:\s]([\d\.-]+\d)', pkg_ver_str)

    if len(res) != 0:
        return res[0]
    else:
        return None


#=======================================
# Get version requirement
def __get_req_ver(obj):
    return shell_exec1("cat %s/data/ver_req | grep %s | awk '{print $2}'" % (sys.path[0], obj))


#=======================================
# Get version in system, method 1
def __get_cur_ver_by_cmd(obj):
    return __parse_pkg_version(shell_exec2("%s --version" % obj))

#=======================================
# Get version in system, method 2
def __get_cur_ver_by_rpm(obj):
    ret = shell_exec3("rpm -qa | grep %s | xargs rpm -q --qf '%%{name}\\t%%{version}\\n'" % obj)

    if 0 != ret[0]:
	return []
    else:
	nvs = ret[1].split('\n')
        return [ [nv.split('\t')[0], __parse_pkg_version(nv.split('\t')[1])] for nv in nvs ]

#=======================================
# Get version in system, method 3
def __get_cur_ver_by_dpkg(obj):
    ret = shell_exec3("dpkg-query -W -f='${Package}\\t${Version}\\n' *%s*" % obj)

    if 0 != ret[0]:
	return []
    else:
	nvs = ret[1].split('\n')
        return [ [nv.split('\t')[0], __parse_pkg_version(nv.split('\t')[1])] for nv in nvs ]
        
#=======================================
# compare version
def compare_ver(cur_ver, req_ver, is_above_req):
    # not allowed None
    if cur_ver == None or req_ver == None:
        return False

    # deal with '-'
    cur_ver = cur_ver.replace('-', '.')
    req_ver = req_ver.replace('-', '.')

    # split 
    cur_vers = cur_ver.split('.')
    req_vers = req_ver.split('.')
    
    # Convert list members to integer type
    cur_vers_int = [ int(num) for num in cur_vers ]
    req_vers_int = [ int(num) for num in req_vers ]

    # Compare 2 lists
    if is_above_req:
	return cur_vers_int >= req_vers_int
    else:
	return cur_vers_int == req_vers_int

#=======================================
# pkg_mgr_version
def xulrunner_version_check():
    
    global info
    
    try:
        # get required version
        vr = __get_req_ver("xulrunner")
    
        # check by xulrunner --version
	c = False
        vc1 = __get_cur_ver_by_cmd("xulrunner")
        c = c | compare_ver(vc1, vr, True);
        __P("xulrunner --version check(%s): current version| %s      required version| %s\n" %(c, vc1, vr))
        if c: return [TETJ_PASS, info]

        # check by rpm
	c = False
        vcg2 = __get_cur_ver_by_rpm("xulrunner")
        for vc2 in vcg2:
            c = c | compare_ver(vc2[1], vr, True);
            __P("xulrunner rpm check(%s): pacakge| %s      current version| %s      required version| %s\n" %(c, vc2[0], vc2[1], vr))
	if c: return [TETJ_PASS, info]

        # check by dpkg
	c = False
        vcg3 = __get_cur_ver_by_dpkg("xulrunner")
        for vc3 in vcg3:
            c = c | compare_ver(vc3[1], vr, True);
            __P("xulrunner dpkg check(%s): package| %s      current version| %s      required version| %s\n" %(c, vc3[0], vc3[1], vr))
	if c: return [TETJ_PASS, info]
                
        return [TETJ_FAIL, info]   
    
    except Exception, Ex:
        print Ex.__str__
        return [TETJ_FAIL, info]   

#=======================================
# test
if __name__ == '__main__':
    p = xulrunner_version_check()
    print "result:%s" % RESULT_CODES[p[0]]
